@@ -4,9 +4,19 @@ namespace XLite\Module\IPAYINTERNATIONAL\MyPOSVirtual\Model\Payment\Processor;
 
 class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
 {
-    public $IPCVersion = '1.0';
+    const PAYMENT_METHOD_CARD = 1;
+    const PAYMENT_METHOD_IDEAL = 2;
+    const PAYMENT_METHOD_ALL = 3;
+
+    const PAYMENT_STATUS_SUCCESS = 1;
+    const PAYMENT_STATUS_PENDING = 2;
+    const PAYMENT_METHOD_ERROR = 3;
+    const PAYMENT_METHOD_EXPIRED = 4;
+
+    public $IPCVersion = '1.4';
     public $IPCLanguage = 'EN';
     public $allowedCurrencies = array('BGN', 'USD', 'EUR', 'GBP', 'CHF', 'JPY', 'RON', 'HRK', 'NOK', 'SEK', 'CZK', 'HUF', 'PLN', 'DKK', 'ISK',);
+    public $iDealCurrencies = array('EUR',);
     public $apiConnectTimeout = 120;
     public $learnMoreUrl = 'https://mypos.com/en/ecommerce';
     public $signUpUrl = 'https://www.mypos.com/en/register';
@@ -43,10 +53,12 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
             $sid = $this->getSetting('test_sid');
             $walletNumber = $this->getSetting('test_wallet_nr');
             $keyIndex = $this->getSetting('test_private_key_idx');
+            $test_prefix = $this->getSetting('mypos_test_prefix');
         } else {
             $sid = $this->getSetting('live_sid');
             $walletNumber = $this->getSetting('live_wallet_nr');
             $keyIndex = $this->getSetting('live_private_key_idx');
+            $test_prefix = null;
         }
         
         $order = $this->transaction->getOrder();
@@ -57,8 +69,8 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
         if (!empty($session->checkoutEmail)) {
             $email = $session->checkoutEmail;
             
-            //because it won't be unset, nor will the 
-            //email input be populated with this value
+            // because it won't be unset, nor will the
+            // email input be populated with this value
             unset($session->checkoutEmail);
         }
         $billingAddress = $profile->getBillingAddress();
@@ -69,10 +81,13 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
             'IPCLanguage' => $this->IPCLanguage,
             'WalletNumber' => $walletNumber,
             'SID' => $sid,
-            
+
+            'PaymentParametersRequired' => $this->getSetting('payment_parameters_required'),
+            'PaymentMethod' => $this->getSetting('payment_method'),
+
             'Amount' => $currency->roundValue($order->getTotal()),
             'Currency' => $currency->getCode(),
-            'OrderID' => $order->getOrderId(),
+            'OrderID' => (!is_null($test_prefix) ? $test_prefix : '') . $order->getOrderId(),
             'URL_OK' => $this->getReturnURL(null, true),
             'URL_Cancel' => $this->getReturnURL(null, true, true),
             'URL_Notify' => $this->getCallbackURL(null, true),
@@ -87,7 +102,7 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
             'CustomerZIPCode' => $billingAddress->getZipcode(),
             'CustomerAddress' => $billingAddress->getStreet(),
             'Note' => $order->getNotes(),
-            'Source' => 'sc_x-cart'// . \XLite::getInstance()->getVersion()
+            'Source' => 'sc_x-cart ' . \XLite::getInstance()->getVersion() . ' plugin version: v1.1.0'
         );
         
         $orderItems = $order->getItems();
@@ -123,7 +138,9 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
                 $formFields
             );
         }
-        
+
+        $order->processSucceed();
+
         return $formFields;
     }
     
@@ -136,9 +153,34 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
      */
     protected function getAllowedCurrencies(\XLite\Model\Payment\Method $method)
     {
-        return $this->allowedCurrencies;
+        if ($this->getSetting('payment_method') != self::PAYMENT_METHOD_IDEAL) {
+            return $this->allowedCurrencies;
+        } else {
+            return $this->iDealCurrencies;
+        }
     }
     
+    /**
+     * Get payment method setting by name
+     *
+     * @param string $name Setting name
+     *
+     * @return mixed|null
+     */
+    protected function getSetting($name)
+    {
+        $result = parent::getSetting($name);
+
+        if (is_null($result)) {
+            $method = \XLite\Core\Database::getRepo('XLite\Model\Payment\Method')->findOneBy(array('service_name' => 'MyPOSVirtual'));
+            $result = $method
+                ? $method->getSetting($name)
+                : null;
+        }
+
+        return $result;
+    }
+
     /**
      * Process callback
      *
@@ -181,9 +223,14 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
             switch ($postData['IPCmethod']) {
                 case 'IPCPurchaseNotify':
                     $currency = $transaction->getCurrency();
-                    
+                    if ($this->getSetting('mode') != 'live'){
+                        $orderId = str_replace($this->getSetting('mypos_test_prefix'), '', $postData['OrderID']);
+                    } else {
+                        $orderId = $postData['OrderID'];
+                    }
+
                     if (($postData['SID'] == $sid)
-                    && ($postData['OrderID'] == $order->getOrderId())
+                    && ($orderId == $order->getOrderId())
                     && ($postData['Amount'] == $currency->roundValue($order->getTotal()))
                     && ($postData['Currency'] == $currency->getCode())
                     ) {
@@ -313,6 +360,7 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
                 );
                 $transaction->setNote($message);
                 $transaction->setStatus($transaction::STATUS_CANCELED);
+                $transaction->registerTransactionInOrderHistory();
 
                 if ($logRequests) {
                     \XLite\Module\IPAYINTERNATIONAL\MyPOSVirtual\Main::addLog(
@@ -378,7 +426,7 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
      */
     public function getCheckoutTemplate(\XLite\Model\Payment\Method $method)
     {
-        return 'modules/IPAYINTERNATIONAL/MyPOSVirtual/checkout/mypos.tpl';
+        return 'modules/IPAYINTERNATIONAL/MyPOSVirtual/checkout/mypos.twig';
     }
     
     /**
@@ -402,6 +450,17 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
      */
     public function isConfigured(\XLite\Model\Payment\Method $method)
     {
+
+        if (!$method->getSetting('mypos_test_prefix')) {
+            $settings = new \XLite\Model\Payment\MethodSetting();
+
+            $settings->setPaymentMethod($method)
+                ->setName('mypos_test_prefix')
+                ->setValue( uniqid() . '_');
+
+            $method->addSettings($settings);
+        }
+
         if ($method->getSetting('mode') != 'live') {
             return parent::isConfigured($method)
                     && $method->getSetting('test_api_url')
@@ -420,7 +479,15 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
                     && $method->getSetting('live_private_key_idx');
         }
     }
-    
+
+    /**
+     * @return bool
+     */
+    function isValidForIdeal()
+    {
+        return in_array(\XLite::getInstance()->getCurrency()->getCode(), $this->iDealCurrencies);
+    }
+
     /**
      * Get payment method icon URL for the checkout page
      *
@@ -429,9 +496,17 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
     public function getCheckoutIconURL()
     {
         $module = $this->getModule();
-        
-        $resourcePath = 'modules/' . $module->getAuthor() . '/' . $module->getName() . '/checkout/checkout_icon.png';
-        
+
+        if ($this->isValidForIdeal() &&  $this->getSetting('payment_method') == self::PAYMENT_METHOD_ALL) {
+            $imageName = 'card_schemes_ideal_no_bg.png';
+        } elseif ($this->isValidForIdeal() &&  $this->getSetting('payment_method') == self::PAYMENT_METHOD_IDEAL) {
+            $imageName = 'mypos_ideal_no_bg.png';
+        } else {
+            $imageName = 'card_schemes_no_bg.png';
+        }
+
+        $resourcePath = 'modules/' . $module->getAuthor() . '/' . $module->getName() . '/checkout/' .  $imageName;
+
         return \XLite\Core\Layout::getInstance()->getResourceWebPath($resourcePath);
     }
     
@@ -531,7 +606,45 @@ class MyPOSVirtual extends \XLite\Model\Payment\Base\WebBased
             return json_decode($apiResponse, true);
         }
     }
-    
+
+    public function checkPaymentStatusApiCall($orderId)
+    {
+        if ($this->getSetting('mode') != 'live') {
+            $sid = $this->getSetting('test_sid');
+            $walletNumber = $this->getSetting('test_wallet_nr');
+            $keyIndex = $this->getSetting('test_private_key_idx');
+        } else {
+            $sid = $this->getSetting('live_sid');
+            $walletNumber = $this->getSetting('live_wallet_nr');
+            $keyIndex = $this->getSetting('live_private_key_idx');
+        }
+
+        $methodParams = array(
+            'IPCmethod' => 'IPCGetPaymentStatus',
+            'KeyIndex' => $keyIndex,
+            'IPCVersion' => $this->IPCVersion,
+            'IPCLanguage' => $this->IPCLanguage,
+            'WalletNumber' => $walletNumber,
+            'SID' => $sid,
+
+            'OrderID' => $orderId,
+
+            'Source' => 'sc_x-cart',
+            'OutputFormat' => 'json'
+        );
+
+        $errorMsg = '';
+        $apiResponse = $this->makeApiCall($methodParams, $errorMsg);
+        if ($apiResponse === false) {
+            return array(
+                'Status' => 1,
+                'StatusMsg' => $errorMsg
+            );
+        } else {
+            return json_decode($apiResponse, true);
+        }
+    }
+
     /**
      * Make a call to the myPOS API
      *
