@@ -27,6 +27,6 @@ class Config extends \XLite\View\AView
      */
     protected function getDefaultTemplate()
     {
-        return 'modules/IPAYINTERNATIONAL/MyPOSVirtual/config.tpl';
+        return 'modules/IPAYINTERNATIONAL/MyPOSVirtual/config.twig';
     }
 }
